#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/11/5 4:36 下午
# @Author  : shuming.wsm
# @Site    : 
# @File    : config.py
# @Software: PyCharm
# @Desc    :

websocket_address = "localhost"
websocket_port = 8101
websocket_path = "ws/simulator"
